@@ -441,6 +441,85 @@ namespace apg {
         : "xmm0", "xmm1", "xmm2", "memory" );
     }
 
+    void hrot_ssse3(uint64_t *cc, uint64_t *nc, const uint8_t *ruledata) {
+        asm (
+        "pxor %%xmm2, %%xmm2 \n\t"
+        "movups 32(%2), %%xmm5 \n\t"
+        "movups 48(%2), %%xmm6 \n\t"
+
+        "movups (%0), %%xmm0 \n\t"
+        "movups (%1), %%xmm1 \n\t"
+        "movups (%2), %%xmm3 \n\t"
+        "movups 16(%2), %%xmm4 \n\t"
+        "pcmpeqb %%xmm2, %%xmm0 \n\t"
+        "movdqa %%xmm6, %%xmm7 \n\t"
+        "pshufb %%xmm1, %%xmm7 \n\t"
+        "psrld $3, %%xmm1 \n\t"
+        "pand %%xmm5, %%xmm1 \n\t"
+        "pshufb %%xmm1, %%xmm3 \n\t"
+        "pshufb %%xmm1, %%xmm4 \n\t"
+        "pand %%xmm3, %%xmm0 \n\t"
+        "pxor %%xmm4, %%xmm0 \n\t"
+        "pand %%xmm7, %%xmm0 \n\t"
+        "pcmpeqb %%xmm2, %%xmm0 \n\t"
+        "movups %%xmm0, (%1) \n\t"
+
+        "movups 16(%0), %%xmm0 \n\t"
+        "movups 16(%1), %%xmm1 \n\t"
+        "movups (%2), %%xmm3 \n\t"
+        "movups 16(%2), %%xmm4 \n\t"
+        "pcmpeqb %%xmm2, %%xmm0 \n\t"
+        "movdqa %%xmm6, %%xmm7 \n\t"
+        "pshufb %%xmm1, %%xmm7 \n\t"
+        "psrld $3, %%xmm1 \n\t"
+        "pand %%xmm5, %%xmm1 \n\t"
+        "pshufb %%xmm1, %%xmm3 \n\t"
+        "pshufb %%xmm1, %%xmm4 \n\t"
+        "pand %%xmm3, %%xmm0 \n\t"
+        "pxor %%xmm4, %%xmm0 \n\t"
+        "pand %%xmm7, %%xmm0 \n\t"
+        "pcmpeqb %%xmm2, %%xmm0 \n\t"
+        "movups %%xmm0, 16(%1) \n\t"
+
+        "movups 32(%0), %%xmm0 \n\t"
+        "movups 32(%1), %%xmm1 \n\t"
+        "movups (%2), %%xmm3 \n\t"
+        "movups 16(%2), %%xmm4 \n\t"
+        "pcmpeqb %%xmm2, %%xmm0 \n\t"
+        "movdqa %%xmm6, %%xmm7 \n\t"
+        "pshufb %%xmm1, %%xmm7 \n\t"
+        "psrld $3, %%xmm1 \n\t"
+        "pand %%xmm5, %%xmm1 \n\t"
+        "pshufb %%xmm1, %%xmm3 \n\t"
+        "pshufb %%xmm1, %%xmm4 \n\t"
+        "pand %%xmm3, %%xmm0 \n\t"
+        "pxor %%xmm4, %%xmm0 \n\t"
+        "pand %%xmm7, %%xmm0 \n\t"
+        "pcmpeqb %%xmm2, %%xmm0 \n\t"
+        "movups %%xmm0, 32(%1) \n\t"
+
+        "movups 48(%0), %%xmm0 \n\t"
+        "movups 48(%1), %%xmm1 \n\t"
+        "movups (%2), %%xmm3 \n\t"
+        "movups 16(%2), %%xmm4 \n\t"
+        "pcmpeqb %%xmm2, %%xmm0 \n\t"
+        "movdqa %%xmm6, %%xmm7 \n\t"
+        "pshufb %%xmm1, %%xmm7 \n\t"
+        "psrld $3, %%xmm1 \n\t"
+        "pand %%xmm5, %%xmm1 \n\t"
+        "pshufb %%xmm1, %%xmm3 \n\t"
+        "pshufb %%xmm1, %%xmm4 \n\t"
+        "pand %%xmm3, %%xmm0 \n\t"
+        "pxor %%xmm4, %%xmm0 \n\t"
+        "pand %%xmm7, %%xmm0 \n\t"
+        "pcmpeqb %%xmm2, %%xmm0 \n\t"
+        "movups %%xmm0, 48(%1) \n\t"
+
+        : /* no output operands -- implicitly volatile */
+        : "r" (cc), "r" (nc), "r" (ruledata)
+        : "xmm0", "xmm1", "xmm2", "xmm3", "xmm4", "xmm5", "xmm6", "xmm7", "memory" );
+    }
+
     void ltl_kernel(uint64_t *inleaves, uint64_t *outleafx, int range, uint8_t bmin, uint8_t bmax, uint8_t smin, uint8_t smax) {
 
         uint64_t lb1 = (bmin ^ smin);
@@ -472,4 +551,22 @@ namespace apg {
         bytes2bits(b, outleafx);
     }
 
+    void hrot_kernel(uint64_t *inleaves, uint64_t *outleafx, int range, const uint8_t *ruledata) {
+
+        uint64_t a[128];
+        uint64_t b[32];
+        uint64_t c[32];
+        uint64_t d[4] = {inleaves[3], inleaves[6], inleaves[9], inleaves[12]};
+        bits2bytes_sse2(inleaves, a);
+        bits2bytes_sse2(inleaves+4, (a+32));
+        bits2bytes_sse2(inleaves+8, (a+64));
+        bits2bytes_sse2(inleaves+12, (a+96));
+        bits2bytes_sse2(d, c);
+        convolve2d(a, b, range);
+        hrot_ssse3(c, b, ruledata);
+        hrot_ssse3(c + 8, b + 8, ruledata);
+        hrot_ssse3(c + 16, b + 16, ruledata);
+        hrot_ssse3(c + 24, b + 24, ruledata);
+        bytes2bits(b, outleafx);
+    }
 }
