@@ -67,11 +67,6 @@ extern "C"
 
     void SavePatternRLE(void *ptr, const char *filename, const char *header, const char *footer) {
 
-        std::cerr << "filename: '" << filename << "'\n";
-        std::cerr << "header: '" << header << "'\n";
-        std::cerr << "footer: '" << footer << "'\n";
-        std::cerr << "--------" << std::endl;
-
         std::ofstream out(filename);
         auto ppat = reinterpret_cast<apg::pattern*>(ptr);
         out << header;
@@ -81,11 +76,6 @@ extern "C"
 
     void SavePatternMC(void *ptr, const char *filename, const char *header, const char *footer) {
 
-        std::cerr << "filename: '" << filename << "'\n";
-        std::cerr << "header: '" << header << "'\n";
-        std::cerr << "footer: '" << footer << "'\n";
-        std::cerr << "--------" << std::endl;
-
         std::ofstream out(filename);
         auto ppat = reinterpret_cast<apg::pattern*>(ptr);
         auto lab = ppat->getlab();
@@ -245,6 +235,12 @@ extern "C"
         s.copy(buffer, 2048);
     }
 
+    void GetWechslerOfPattern(void* pat, char* buffer) {
+        auto ppat = reinterpret_cast<apg::pattern*>(pat);
+        std::string s = ppat->phase_wechsler();
+        s.copy(buffer, 2048);
+    }
+
     void GetApgcodeOfPattern(void* pat, char* buffer) {
         auto ppat = reinterpret_cast<apg::pattern*>(pat);
         std::string s = ppat->apgcode();
