@@ -441,14 +441,14 @@ namespace apg {
                     if (posrule != std::string::npos) {
                         rule = sanirule(line.substr(posrule + 4));
                         delete grm; grm = getMapper(rule);
-                        std::cerr << line << " interpreted as " << rule << std::endl;
+                        // std::cerr << line << " interpreted as " << rule << std::endl;
                     }
                 } else if (line[0] == '#') {
                     if (line.length() >= 2 && (line[1] == 'R' || line[1] == 'G')) {
                         if (line[1] == 'R') {
                             rule = sanirule(line.substr(3));
                             delete grm; grm = getMapper(rule);
-                            std::cerr << line << " interpreted as " << rule << std::endl;
+                            // std::cerr << line << " interpreted as " << rule << std::endl;
                         }
                     } else if ((line.length() > 4) && (line.substr(0, 4) == "#CLL")) {
                         // Lifelib-specific meta-information
