@@ -581,6 +581,10 @@ namespace apg {
             }
         }
 
+        std::string phase_wechsler() {
+            return phase_wechsler(rule_symmetries());
+        }
+
         std::string apgcode() {
             /*
             * Returns the apgcode of a still-life, oscillator or spaceship:
