@@ -30,9 +30,13 @@ class Pattern(object):
         self.session = session
         self.owner = owner
 
+        session.pattern_ptrs.add(ptr)
+
     def __del__(self):
 
-        self.lifelib('DeletePattern', self.ptr)
+        if self.ptr in self.session.pattern_ptrs:
+            self.lifelib('DeletePattern', self.ptr)
+            self.session.pattern_ptrs.remove(self.ptr)
 
     def __repr__(self):
 
@@ -371,6 +375,10 @@ class Pattern(object):
 
         return self.lifelib('GetRuleOfPattern', self.ptr, [2048])
 
+    def phase_wechsler(self):
+
+        return self.lifelib('GetWechslerOfPattern', self.ptr, [2048])
+
     def oscar(self, maxexp=24, eventual_oscillator=True, verbose=True, return_apgcode=False):
 
         logdiam = self.lifelib('GetDiameterOfPattern', self.ptr)
@@ -380,7 +388,7 @@ class Pattern(object):
 
         x = self
         for exponent in range(8, maxexp+2, 2):
-            if (verbose):
+            if (verbose) and (exponent > 8):
                 print("Checking periodicity with exponent %d..." % exponent)
             y = x.pdetect_or_advance(exponent)
             if isinstance(y, dict):
@@ -669,6 +677,11 @@ class Pattern(object):
         populations = [self.lifelib('GetPopulationOfPattern', self.ptr, p) for p in relprimes]
         return chinese_remainder(relprimes, populations)
 
+    @property
+    def wechsler(self):
+
+        return self.phase_wechsler()
+
     @property
     def apgcode(self):
 
