@@ -3,6 +3,7 @@ from .pattern import Pattern
 from .samples import download_samples as dsamples
 from .samples import validate_symmetry
 import os
+import atexit
 
 from ..genera import sanirule
 
@@ -22,6 +23,8 @@ class Lifetree(object):
         self.ptr = lifelib('CreateLifetree', memory, n_layers)
         self.n_layers = n_layers
 
+        session.lifetree_ptrs.add(self.ptr)
+
     def load(self, filename, compressed='deduce', tempfile='tempfile'):
 
         filename = os.path.abspath(filename)
@@ -110,7 +113,35 @@ class Lifetree(object):
 
     def __del__(self):
 
-        self.lifelib('DeleteLifetree', self.ptr)
+        if self.ptr in self.session.lifetree_ptrs:
+            self.lifelib('DeleteLifetree', self.ptr)
+            self.session.lifetree_ptrs.remove(self.ptr)
+
+all_sessions = []
+
+def cleanup_sessions():
+
+    # Deallocate patterns
+    for sess in all_sessions:
+        for ptr in sess.pattern_ptrs:
+            sess.lifelib('DeletePattern', ptr)
+        sess.pattern_ptrs = set([])
+
+    # Deallocate lifetrees
+    for sess in all_sessions:
+        for ptr in sess.lifetree_ptrs:
+            sess.lifelib('DeleteLifetree', ptr)
+        sess.lifetree_ptrs = set([])
+
+    # Wait for child processes
+    for sess in all_sessions:
+        sess.lifelib.annihilate()
+
+    # Remove references
+    while all_sessions:
+        all_sessions.pop()
+
+atexit.register(cleanup_sessions)
 
 class Session(object):
 
@@ -128,6 +159,10 @@ class Session(object):
         self.default_planes = default_planes
         self.rules = list(rules)
         self.lifelib = WrappedLibrary(soname, local_bash=local_bash, local_python=local_python)
+        self.lifetree_ptrs = set([])
+        self.pattern_ptrs = set([])
+
+        all_sessions.append(self)
 
     def lifetree(self, *args, **kwargs):
 
